#include <iostream>
#include <stdexcept>

#include "unittest/unittest.h"

using namespace unittest;

template < typename T >
struct UnitTestAssert
:
  public TestSuite
{
    struct AssertEqualDetails
    {
      void operator()()
      {
        T e = 42, a = e;
        
        UNITTEST_ASSERT(Equal, e, a)("prefix")(42.42)("postfix");
        ++a;
        
        try
        {
          UNITTEST_ASSERT(Equal, e, a)("prefix")(42.42)("postfix");
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const & rc_ex)
        {
          //nop
        } /* end of catch */
      }
    };

    struct AssertEqual
    {
      void operator()()
      {
        T e = 42, a = e;
        
        UNITTEST_ASSERT(Equal, e, a);
        ++a;
        
        try
        {
          UNITTEST_ASSERT(Equal, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertNotEqual
    {
      void operator()()
      {
        T e = 42, a = e - 1;
        
        UNITTEST_ASSERT(NotEqual, e, a);
        ++a;
        
        try
        {
          UNITTEST_ASSERT(NotEqual, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertLess
    {
      void operator()()
      {
        T e = 42, a = e - 1;
        
        UNITTEST_ASSERT(Less, e, a);
        ++a;
        
        try
        {
          UNITTEST_ASSERT(Less, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertLessEqual
    {
      void operator()()
      {
        T e = 42, a = e - 1;
        
        UNITTEST_ASSERT(LessEqual, e, a);
        ++a;
        UNITTEST_ASSERT(LessEqual, e, a);
        ++a;
        
        try
        {
          UNITTEST_ASSERT(LessEqual, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertGreater
    {
      void operator()()
      {
        T e = 42, a = e + 1;
        
        UNITTEST_ASSERT(Greater, e, a);
        --a;
        
        try
        {
          UNITTEST_ASSERT(Greater, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertGreaterEqual
    {
      void operator()()
      {
        T e = 42, a = e + 1;
        
        UNITTEST_ASSERT(GreaterEqual, e, a);
        --a;
        UNITTEST_ASSERT(GreaterEqual, e, a);
        --a;
        
        try
        {
          UNITTEST_ASSERT(GreaterEqual, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    
  UnitTestAssert()
  {
    add(TestBasePtr(new Test<AssertEqualDetails>));
    add(TestBasePtr(new Test<AssertEqual>));
    add(TestBasePtr(new Test<AssertNotEqual>));
    add(TestBasePtr(new Test<AssertLess>));
    add(TestBasePtr(new Test<AssertLessEqual>));
    add(TestBasePtr(new Test<AssertGreater>));
    add(TestBasePtr(new Test<AssertGreaterEqual>));
  }
};

static AutoRegister < UnitTestAssert<char> > ts_char;
static AutoRegister < UnitTestAssert<signed char> > ts_schar;
static AutoRegister < UnitTestAssert<unsigned char> > ts_uchar;
static AutoRegister < UnitTestAssert<short> > ts_short;
static AutoRegister < UnitTestAssert<unsigned short> > ts_sshort;
static AutoRegister < UnitTestAssert<int> > ts_int;
static AutoRegister < UnitTestAssert<unsigned int> > ts_sint;
static AutoRegister < UnitTestAssert<long> > ts_long;
static AutoRegister < UnitTestAssert<unsigned long> > ts_ulong;
