#include <iostream>
#include <stdexcept>

#include "unittest/unittest.h"

using namespace unittest;

struct UnitTestAssertString
:
  public TestSuite
{
    struct AssertEqual
    {
      void operator()()
      {
        std::string e = std::string("hello world"), a = e;
        
        UNITTEST_ASSERT(Equal, e, a);
        a += "!";
        
        try
        {
          UNITTEST_ASSERT(Equal, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertNotEqual
    {
      void operator()()
      {
        std::string e = std::string("hello world"), a = e.substr(0, e.size() - 1);
        
        UNITTEST_ASSERT(NotEqual, e, a);
        a += "d";
        
        try
        {
          UNITTEST_ASSERT(NotEqual, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertLess
    {
      void operator()()
      {
        std::string e = std::string("hello world"), a = e.substr(0, e.size() - 1);
        
        UNITTEST_ASSERT(Less, e, a);
        a += "d";
        
        try
        {
          UNITTEST_ASSERT(Less, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertLessEqual
    {
      void operator()()
      {
        std::string e = std::string("hello world"), a = e.substr(0, e.size() - 1);
        
        UNITTEST_ASSERT(LessEqual, e, a);
        a += "d";
        UNITTEST_ASSERT(LessEqual, e, a);
        a += "!";
        
        try
        {
          UNITTEST_ASSERT(LessEqual, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertGreater
    {
      void operator()()
      {
        std::string e = std::string("hello world"), a = e + "!";
        
        UNITTEST_ASSERT(Greater, e, a);
        a = a.substr(0, a.size() - 1);
        
        try
        {
          UNITTEST_ASSERT(Greater, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    struct AssertGreaterEqual
    {
      void operator()()
      {
        std::string e = std::string("hello world"), a = e + "!";
        
        UNITTEST_ASSERT(GreaterEqual, e, a);
        a = a.substr(0, a.size() - 1);
        UNITTEST_ASSERT(GreaterEqual, e, a);
        a = a.substr(0, a.size() - 1);
        
        try
        {
          UNITTEST_ASSERT(GreaterEqual, e, a);
          throw std::logic_error("unreachable code");
        } /* end of try */
        catch (Exception const &)
        {
          //nop
        } /* end of catch */
      }
    };
    
    
  UnitTestAssertString()
  {
    add(TestBasePtr(new Test<AssertEqual>));
    add(TestBasePtr(new Test<AssertNotEqual>));
    add(TestBasePtr(new Test<AssertLess>));
    add(TestBasePtr(new Test<AssertLessEqual>));
    add(TestBasePtr(new Test<AssertGreater>));
    add(TestBasePtr(new Test<AssertGreaterEqual>));
  }
};

static AutoRegister < UnitTestAssertString > ts;
