
#include <memory>
#include <sstream>

#include "unittest/unittest.h"


using namespace unittest;


struct UnitTestSelect
:
  public TestSuite
{
    struct TestOk
    {
      void operator()()
      {
      }
    };
    
    struct CreateTests
    {
      CreateTests()
      :
        m_tests("A")
      {
        std::auto_ptr < TestSuite > a_ts1;
        std::auto_ptr < TestSuite > a_ts2;
        
        a_ts1 = std::auto_ptr < TestSuite >(new TestSuite("B"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "a"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "b"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "c"));
        m_tests.add(a_ts1.release());
        
        a_ts1 = std::auto_ptr < TestSuite >(new TestSuite("C"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "d"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "e"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "f"));
        m_tests.add(a_ts1.release());
        
        a_ts1 = std::auto_ptr < TestSuite >(new TestSuite("D"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "g"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "h"));
        a_ts1->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "i"));
        m_tests.add(a_ts1.release());
        
        a_ts1 = std::auto_ptr < TestSuite >(new TestSuite("E"));
        
        a_ts2 = std::auto_ptr < TestSuite >(new TestSuite("F"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "j"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "k"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "l"));
        a_ts1->add(a_ts2.release());
        
        a_ts2 = std::auto_ptr < TestSuite >(new TestSuite("G"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "m"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "n"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "o"));
        a_ts1->add(a_ts2.release());
        
        a_ts2 = std::auto_ptr < TestSuite >(new TestSuite("H"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "p"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "q"));
        a_ts2->add(new Test < voidfunc > (CreateTests::dummyTestFunction, "r"));
        a_ts1->add(a_ts2.release());
        
        m_tests.add(a_ts1.release());
        
        a_ts1 = std::auto_ptr < TestSuite >(new TestSuite("I"));
        a_ts1->add(new Test < TestOk > ("s"));
        
        m_tests.add(a_ts1.release());
      }
      
      ~CreateTests()
      {
      }
      
      static void dummyTestFunction(void)
      {
      }
      
      TestSuite                  m_tests;
      std::vector< std::string > m_includeList;
      std::vector< std::string > m_excludeList;
      std::stringstream          m_ss;
    };
    
    
    struct TestExecutor
    :
      public virtual OstreamVisitor,
      public virtual Executor
    {
        TestExecutor ( std::ostream & ostream )
      :
        OstreamVisitor(ostream)
      {
      }
      
      void operator()(TestBase & test) const
      {
        ostream << test.name();
      }
      
      void operator()(TestSuite & suite) const
      {
        ostream << suite.name();
        Executor::operator()(suite);
      }
    };
    
    
    struct selectAll : public CreateTests
    {
      void operator()()
      {
        TestExecutor executor(m_ss);
        
        SelectSuite ss("A", m_includeList, m_excludeList);
        ss(m_tests);
        
        executor(ss);
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "ABabcCdefDghiEFjklGmnoHpqrIs"
          ),
          m_ss.str()
        );
      }
    };
    
    struct selectInclude : public CreateTests
    {
      void operator()()
      {
        TestExecutor executor(m_ss);
        
        m_includeList.push_back("B");
        m_includeList.push_back("D");
        m_includeList.push_back("G");
        
        SelectSuite ss("A", m_includeList, m_excludeList);
        ss(m_tests);
        
        executor(ss);
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "ABabcDghiEGmno"
          ),
          m_ss.str()
        );
      }
    };

    struct selectExclude : public CreateTests
    {
      void operator()()
      {
        TestExecutor executor(m_ss);
        
        m_includeList.push_back("B");
        m_includeList.push_back("E");
        m_excludeList.push_back("G");
        m_excludeList.push_back("b");
        m_excludeList.push_back("q");
        
        SelectSuite ss("A", m_includeList, m_excludeList);
        ss(m_tests);
        
        executor(ss);
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "ABacEFjklHpr"
          ),
          m_ss.str()
        );
      }
    };

    UnitTestSelect(){
      add(new Test<selectAll>);
      add(new Test<selectInclude>);
      add(new Test<selectExclude>);
    }
};

static AutoRegister<UnitTestSelect> ts;
