#include <iostream>
#include <fstream>
#include <stdexcept>
#include <string>
#include <vector>

#include "unittest/unittest.h"


using namespace unittest;


int main(int argc, char **argv)
{
  std::sort(
    TestSuite::instance().tests.begin(),
    TestSuite::instance().tests.end(),
    NameCompare()
  );
  
  std::vector< std::string > includeList;
  std::vector< std::string > excludeList;
  std::string xmlfile;
  int verbose = 0;
  bool listtests = false;
  bool help = false;
  
  for (int cnt = 1, max = argc; cnt < max; ++cnt)
  {
    std::string arg(argv[cnt]);
    if(arg == "-l" || arg == "--listtests")
    {
      listtests = true;
      break;
    }
    if(arg == "-h" || arg == "--help")
    {
      help = true;
      break;
    }
    if(arg == "-v" || arg == "--verbose")
    {
      if(cnt + 1 < argc)
      {
        std::istringstream ss(argv[cnt + 1]);
        ss >> verbose;
        if(ss.bad())
        {
          ++verbose;
        }
        else
        {
          ++cnt;
        }
      }
      else
      {
        ++verbose;
      }
      continue;
    }
    else if(arg == "-e" || arg == "--exclude")
    {
      ++cnt;
      if(cnt < argc)
      {
        excludeList.push_back(argv[cnt]);
      }
      continue;
    }
    else if(arg == "-x" || arg == "--xml")
    {
      ++cnt;
      if(cnt >= argc)
      {
        xmlfile = "tests.xml";
      }
      else
      {
        xmlfile = argv[cnt];
      }
    }
    else if(arg == "all")
    {
      includeList.clear();
      continue;
    }
    else
    {
      includeList.push_back(argv[cnt]);
    }
  } /* end of for */
  
  if(listtests)
  {
    std::cout << "Available tests:" << std::endl;
    ListVisitor listVisitor(std::cout);
    listVisitor(TestSuite::instance());
    exit(0);
  }
  else if(help)
  {
    std::cout << "USAGE: " << argv[0] << " [-h/--help] [-x/--xml <xml file name>] [-v/--verbose [n]] [-l/--listtests] [[-e/--exclude testname]] testname [[testname]]" << std::endl;
    std::cout << std::endl;
    exit(1);
  }
  
  if(verbose >= 2)
  {
    if(!includeList.empty())
    {
      std::cout << "including into test:" << std::endl;
      for(size_t cnt = 0; cnt < includeList.size(); ++cnt)
      {
        std::cout << "  " << includeList[cnt] << std::endl;
      }
    }
    if(!excludeList.empty())
    {
      std::cout << "excluding from test:" << std::endl;
      for(size_t cnt = 0; cnt < excludeList.size(); ++cnt)
      {
        std::cout << "  " << excludeList[cnt] << std::endl;
      }
    }
  }
  
  SelectSuite ss("all", includeList, excludeList);
  ss(TestSuite::instance());
  
  if(verbose >= 3)
  {
    VerboseExecutor executor(std::cerr);
    executor(ss);
  }
  else if(verbose >= 0)
  {
    TerseExecutor executor(std::cerr);
    executor(ss);
  }
  else
  {
    Executor executor;
    executor(ss);
  }
  
  Counter counter;
  counter(ss);
  
  // print summary if quiet
  if(verbose == -1)
  {
    counter.summary(std::cerr);
  }
  
  if(counter.failures + counter.errors > 0 || (verbose >= 2))
  {
    std::cout << std::endl;
    ResultVisitor resultVisitor(std::cout, (verbose >= 1));
    resultVisitor(ss);
  }
  
  if(!xmlfile.empty())
  {
    std::ofstream file(xmlfile.c_str());
    if(file.good())
    {
      std::cout << "writing test results to file \"" << xmlfile << "\"." << std::endl;
      JunitXmlResultVisitor resultVisitor(file);
      resultVisitor(ss);
    }
    else
    {
      std::cerr << "opening file \"" << xmlfile << "\" for writing failed!" << std::endl;
    }
  }
  
  std::cout << std::endl;
  return (counter.failures + counter.errors);
}
